/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { Context } from './context';
export interface GitHubRelease {
    id: number;
    tag_name: string;
    html_url: string;
    assets: Array<string>;
}
export interface BuildxOpts {
    context: Context;
    standalone?: boolean;
}
export declare class Buildx {
    private readonly context;
    private _version;
    standalone: boolean;
    constructor(opts: BuildxOpts);
    getCommand(args: Array<string>): {
        command: string;
        args: string[];
    };
    isAvailable(): Promise<boolean>;
    printInspect(name: string): Promise<void>;
    get version(): Promise<string | undefined>;
    printVersion(): Promise<void>;
    static parseVersion(stdout: string): string;
    versionSatisfies(range: string, version?: string): Promise<boolean>;
    getBuildImageIDFilePath(): string;
    getBuildMetadataFilePath(): string;
    getBuildImageID(): string | undefined;
    getBuildMetadata(): string | undefined;
    getDigest(): string | undefined;
    generateBuildSecretString(kvp: string): string;
    generateBuildSecretFile(kvp: string): string;
    generateBuildSecret(kvp: string, file: boolean): string;
    getProvenanceInput(name: string): string;
    getProvenanceAttrs(input: string): string;
    static getRelease(version: string): Promise<GitHubRelease>;
    static hasLocalExporter(exporters: string[]): boolean;
    static hasTarExporter(exporters: string[]): boolean;
    static hasDockerExporter(exporters: string[], load?: boolean): boolean;
    static hasExporterType(name: string, exporters: string[]): boolean;
    static hasGitAuthTokenSecret(secrets: string[]): boolean;
}
