"use strict";
/**
 * Copyright 2023 actions-toolkit authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Buildx = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const core = __importStar(require("@actions/core"));
const exec = __importStar(require("@actions/exec"));
const httpm = __importStar(require("@actions/http-client"));
const sync_1 = require("csv-parse/sync");
const semver = __importStar(require("semver"));
const docker_1 = require("./docker");
class Buildx {
    constructor(opts) {
        var _a;
        this.context = opts.context;
        this.standalone = (_a = opts === null || opts === void 0 ? void 0 : opts.standalone) !== null && _a !== void 0 ? _a : !docker_1.Docker.isAvailable();
    }
    getCommand(args) {
        return {
            command: this.standalone ? 'buildx' : 'docker',
            args: this.standalone ? args : ['buildx', ...args]
        };
    }
    isAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            const cmd = this.getCommand([]);
            return yield exec
                .getExecOutput(cmd.command, cmd.args, {
                ignoreReturnCode: true,
                silent: true
            })
                .then(res => {
                if (res.stderr.length > 0 && res.exitCode != 0) {
                    return false;
                }
                return res.exitCode == 0;
            })
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                .catch(error => {
                return false;
            });
        });
    }
    printInspect(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const cmd = this.getCommand(['inspect', name]);
            yield exec.exec(cmd.command, cmd.args, {
                failOnStdErr: false
            });
        });
    }
    get version() {
        return (() => __awaiter(this, void 0, void 0, function* () {
            if (!this._version) {
                const cmd = this.getCommand(['version']);
                this._version = yield exec
                    .getExecOutput(cmd.command, cmd.args, {
                    ignoreReturnCode: true,
                    silent: true
                })
                    .then(res => {
                    if (res.stderr.length > 0 && res.exitCode != 0) {
                        throw new Error(res.stderr.trim());
                    }
                    return Buildx.parseVersion(res.stdout.trim());
                });
            }
            return this._version;
        }))();
    }
    printVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const cmd = this.getCommand(['version']);
            yield exec.exec(cmd.command, cmd.args, {
                failOnStdErr: false
            });
        });
    }
    static parseVersion(stdout) {
        const matches = /\sv?([0-9a-f]{7}|[0-9.]+)/.exec(stdout);
        if (!matches) {
            throw new Error(`Cannot parse buildx version`);
        }
        return matches[1];
    }
    versionSatisfies(range, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const ver = version !== null && version !== void 0 ? version : (yield this.version);
            if (!ver) {
                return false;
            }
            return semver.satisfies(ver, range) || /^[0-9a-f]{7}$/.exec(ver) !== null;
        });
    }
    getBuildImageIDFilePath() {
        return path_1.default.join(this.context.tmpDir(), 'iidfile').split(path_1.default.sep).join(path_1.default.posix.sep);
    }
    getBuildMetadataFilePath() {
        return path_1.default.join(this.context.tmpDir(), 'metadata-file').split(path_1.default.sep).join(path_1.default.posix.sep);
    }
    getBuildImageID() {
        const iidFile = this.getBuildImageIDFilePath();
        if (!fs_1.default.existsSync(iidFile)) {
            return undefined;
        }
        return fs_1.default.readFileSync(iidFile, { encoding: 'utf-8' }).trim();
    }
    getBuildMetadata() {
        const metadataFile = this.getBuildMetadataFilePath();
        if (!fs_1.default.existsSync(metadataFile)) {
            return undefined;
        }
        const content = fs_1.default.readFileSync(metadataFile, { encoding: 'utf-8' }).trim();
        if (content === 'null') {
            return undefined;
        }
        return content;
    }
    getDigest() {
        const metadata = this.getBuildMetadata();
        if (metadata === undefined) {
            return undefined;
        }
        const metadataJSON = JSON.parse(metadata);
        if (metadataJSON['containerimage.digest']) {
            return metadataJSON['containerimage.digest'];
        }
        return undefined;
    }
    generateBuildSecretString(kvp) {
        return this.generateBuildSecret(kvp, false);
    }
    generateBuildSecretFile(kvp) {
        return this.generateBuildSecret(kvp, true);
    }
    generateBuildSecret(kvp, file) {
        const delimiterIndex = kvp.indexOf('=');
        const key = kvp.substring(0, delimiterIndex);
        let value = kvp.substring(delimiterIndex + 1);
        if (key.length == 0 || value.length == 0) {
            throw new Error(`${kvp} is not a valid secret`);
        }
        if (file) {
            if (!fs_1.default.existsSync(value)) {
                throw new Error(`secret file ${value} not found`);
            }
            value = fs_1.default.readFileSync(value, { encoding: 'utf-8' });
        }
        const secretFile = this.context.tmpName({ tmpdir: this.context.tmpDir() });
        fs_1.default.writeFileSync(secretFile, value);
        return `id=${key},src=${secretFile}`;
    }
    getProvenanceInput(name) {
        const input = core.getInput(name);
        if (!input) {
            // if input is not set returns empty string
            return input;
        }
        const builderID = this.context.provenanceBuilderID;
        try {
            return core.getBooleanInput(name) ? `builder-id=${builderID}` : 'false';
        }
        catch (err) {
            // not a valid boolean, so we assume it's a string
            return this.getProvenanceAttrs(input);
        }
    }
    getProvenanceAttrs(input) {
        if (!input) {
            return `builder-id=${this.context.provenanceBuilderID}`;
        }
        // parse attributes from input
        const fields = (0, sync_1.parse)(input, {
            relaxColumnCount: true,
            skipEmptyLines: true
        })[0];
        // check if builder-id attribute exists in the input
        for (const field of fields) {
            const parts = field
                .toString()
                .split(/(?<=^[^=]+?)=/)
                .map(item => item.trim());
            if (parts[0] == 'builder-id') {
                return input;
            }
        }
        // if not add builder-id attribute
        return `${input},builder-id=${this.context.provenanceBuilderID}`;
    }
    static getRelease(version) {
        return __awaiter(this, void 0, void 0, function* () {
            // FIXME: Use https://raw.githubusercontent.com/docker/actions-toolkit/main/.github/buildx-releases.json when repo public
            const url = `https://raw.githubusercontent.com/docker/buildx/master/.github/releases.json`;
            const http = new httpm.HttpClient('docker-actions-toolkit');
            const resp = yield http.get(url);
            const body = yield resp.readBody();
            const statusCode = resp.message.statusCode || 500;
            if (statusCode >= 400) {
                throw new Error(`Failed to get Buildx release ${version} from ${url} with status code ${statusCode}: ${body}`);
            }
            const releases = JSON.parse(body);
            if (!releases[version]) {
                throw new Error(`Cannot find Buildx release ${version} in ${url}`);
            }
            return releases[version];
        });
    }
    static hasLocalExporter(exporters) {
        return Buildx.hasExporterType('local', exporters);
    }
    static hasTarExporter(exporters) {
        return Buildx.hasExporterType('tar', exporters);
    }
    static hasDockerExporter(exporters, load) {
        return load !== null && load !== void 0 ? load : Buildx.hasExporterType('docker', exporters);
    }
    static hasExporterType(name, exporters) {
        const records = (0, sync_1.parse)(exporters.join(`\n`), {
            delimiter: ',',
            trim: true,
            columns: false,
            relaxColumnCount: true
        });
        for (const record of records) {
            if (record.length == 1 && !record[0].startsWith('type=')) {
                // Local if no type is defined
                // https://github.com/docker/buildx/blob/d2bf42f8b4784d83fde17acb3ed84703ddc2156b/build/output.go#L29-L43
                return name == 'local';
            }
            for (const [key, value] of record.map(chunk => chunk.split('=').map(item => item.trim()))) {
                if (key == 'type' && value == name) {
                    return true;
                }
            }
        }
        return false;
    }
    static hasGitAuthTokenSecret(secrets) {
        for (const secret of secrets) {
            if (secret.startsWith('GIT_AUTH_TOKEN=')) {
                return true;
            }
        }
        return false;
    }
}
exports.Buildx = Buildx;
//# sourceMappingURL=buildx.js.map